/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.portlet.cc;

import hu.midori.kosmos.model.CcProject;
import hu.midori.kosmos.model.ModelUtils;
import hu.midori.kosmos.portlet.AbstractKosmosPortlet;
import hu.midori.kosmos.portlet.util.PortletUtils;
import hu.midori.kosmos.protocol.CcService;

import java.net.MalformedURLException;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.caucho.hessian.client.HessianProxyFactory;

/**
 * Monitors <i>CruiseControl</i> continuous integration projects.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class CcMonitoringPortlet extends AbstractKosmosPortlet {
    private final static Log log = LogFactory.getLog(CcMonitoringPortlet.class);
    
    private List<String> monitoredUrls = null;

    @Override
    public void init(PortletConfig config) throws PortletException {
        log.info("Initializing...");
        super.init(config);
        
        monitoredUrls = PortletUtils.breakCommaSeparatedString(config.getInitParameter("monitored.urls"));        
        if(monitoredUrls == null)
            throw new IllegalStateException("'monitored.urls' was not specified in the init-params");
    }
    
    @Override
    public void destroy() {
        log.info("Destroying...");
    }
    
    @Override
    public void doView(RenderRequest request, RenderResponse response) {
        log.info("Viewing...");   

        // connect to web-service
        CcService service = null;                
        HessianProxyFactory factory = new HessianProxyFactory();
        try {
            service = (CcService)factory.create(CcService.class, serviceUrl);
        } catch (MalformedURLException ex) {
            log.error("Unable to connect to the service", ex);
        }

        // get project list if the service was successfully set-up
        Date timestamp = null;
        List<CcProject> projects = null;
        if(service != null) {
            for(String monitoredDir : monitoredUrls) {
                if(projects == null)
                    projects = service.getProjects(monitoredDir);
                else
                    projects.addAll(service.getProjects(monitoredDir));
            }
            log.debug("CC projects downloaded: " + projects.size());

            timestamp = service.getTimestamp();
            log.debug("CC timestamp: " + timestamp);
        }

        // render response
        getPortletContext().setAttribute("timestamp", timestamp);
        String id = request.getParameter("id");    
        if(id != null) {
            // show details
            CcProject project = (CcProject)ModelUtils.findInCollection(projects, id);
            if(project != null) {
                getPortletContext().setAttribute("project", project);        
                getPortletContext().removeAttribute("projects");
            
                renderJsp(request, response, "/pages/cc_monitoring_test_details.jsp", String.format("%s [%s / %s]", getResourceBundle(Locale.getDefault()).getString("portlet.title"), monitoredResource, project.getName()));
                return;
            } else {
            	log.error(String.format("Project \"%s\" not found", id));            	
            }
        }

        // show the full list
        getPortletContext().removeAttribute("project");
        getPortletContext().setAttribute("projects", projects);    
    
        renderJsp(request, response, "/pages/cc_monitoring.jsp", String.format("%s [%s - %d]", getResourceBundle(Locale.getDefault()).getString("portlet.title"), monitoredResource, projects.size()));
    }
    
    @Override
    public void doHelp(RenderRequest request, RenderResponse response) {
        log.info("Helping...");   

        // render response
        renderJsp(request, response, "/pages/cc_monitoring_help.jsp", String.format("%s [%s]", getResourceBundle(Locale.getDefault()).getString("portlet.title"), monitoredResource));
    }
}
