/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.cc;

import hu.midori.kosmos.model.CcBuildStatus;
import hu.midori.kosmos.model.CcProject;
import hu.midori.kosmos.model.CcTest;
import hu.midori.kosmos.protocol.CcService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class CcServiceTest extends AbstractKosmosServiceTest {
    private CcService service = new CcServiceImpl();

    /** Tests finding the most recent logfile. */
    public void testMostRecentFinder() {
        // TODO
    }

    /** Tests with an invalid path. */
    public void testInvalidPath() {
        assertEquals(0, service.getProjects("nosuchpath").size());
    }

    public void testStandardLogs() {// TODO add these files to the repo and the src distro
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/ais");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/cargo-m2-trunk");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/commons-math");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/frifinans");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/gridportlets");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/gridsphere");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/gvlam");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/jboss-3.2");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/jboss-remoting");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/log4rss");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/mobicents-dailytckrun");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/mock-apis-trunk");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/openpacs");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/scarab-mysql");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/scarab-pgsql");
        testLog("/!Usr/Aron/Own/midori/projects/kosmos/src-repo/testdata/cc-log/weta-dfs");
    }

    // TODO add more logfiles

    /** Tests a dir. */
    protected void testLog(String dir) {
        List<CcProject> projects = service.getProjects(dir);
        assertEquals(1, projects.size()); // one CC log contains exactly one project

        // test project
        CcProject project = projects.get(0);
        assertTrue(project.getName().length() > 0);
        assertTrue(project.getBuildLabel().length() > 0);
        assertTrue((project.getStatus() == CcBuildStatus.SUCCEEDED) || (project.getStatus() == CcBuildStatus.FAILED));
        assertTrue(project.getTimestamp().getTime() > 0);
        assertTrue(project.getModifications() >= 0);
        assertTrue(project.getBuildTime().length() > 0);
        assertTrue(project.getTests() >= 0);
        assertTrue(project.getTestsSucceded() >= 0);
        assertTrue(project.getFailures() >= 0);
        assertTrue(project.getErrors() >= 0);
        assertEquals(project.getTests(), project.getTestsSucceded() + project.getFailures() + project.getErrors());
        assertEquals(project.getTests(), project.getTestCases().size());
        assertFalse(StringUtils.isBlank(project.getTestsPerCategoryChartLocator().getLoresUrl()));
        assertTrue((project.getSuccessRate() >= 0.0) && (project.getSuccessRate() <= 1.0));
        assertTrue(project.getLastBuildAge() >= 0);

        // test testcases
        for(CcTest testCase : project.getTestCases()) {
            assertTrue(testCase.getResult() >= 0);
            assertFalse(StringUtils.isBlank(testCase.getClassName()));
            assertFalse(StringUtils.isBlank(testCase.getTestCaseName()));
        }
    }
}
