/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.jira;

import hu.midori.kosmos.model.JiraProject;
import hu.midori.kosmos.protocol.JiraService;
import hu.midori.kosmos.server.AbstractKosmosServiceTest;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

/**
 * Unit test.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class JiraServiceTest extends AbstractKosmosServiceTest {
    private JiraService service = new JiraWebCrawlingServiceImpl();

    /** Tests with invalid URL. */
    public void testInvalidUrl() {
        assertEquals(0, service.getProjects("http://www.nosuchdomain.com/jira").size());
    }

    /** Tests accessing secured databases. */
    public void testAuthentication() {
        // test secured without signing in
        assertEquals(0, service.getProjects("http://svn.jboss.com/jbossas").size());// TODO specify secured here

        // test secured with signing in
        testDatabase("http://myusername:mypassword@xxx.yyy.zzz/myproject");// TODO
    }

    public void testApacheInstance() {
        // TODO pass Apache URL to testInstance();
    }

    public void testAtlassianInstance() {
        testDatabase("http://jira.atlassian.com/browse/COBJ");
        testDatabase("http://jira.atlassian.com/browse/CORE");
        testDatabase("http://jira.atlassian.com/browse/MAIL");
        testDatabase("http://jira.atlassian.com/browse/PLUG");
        testDatabase("http://jira.atlassian.com/browse/CONF");
        testDatabase("http://jira.atlassian.com/browse/JRA");
        testDatabase("http://jira.atlassian.com/browse/PERF");
        testDatabase("http://jira.atlassian.com/browse/SVN");
        testDatabase("http://jira.atlassian.com/browse/TST");
    }

    public void testCodeHausInstance() {
        testDatabase("http://jira.codehaus.org/browse/ACTIVEIO");
        testDatabase("http://jira.codehaus.org/browse/AWS");
        testDatabase("http://jira.codehaus.org/browse/ASP");
        testDatabase("http://jira.codehaus.org/browse/AJLIB");
        testDatabase("http://jira.codehaus.org/browse/ANNOGEN");
        testDatabase("http://jira.codehaus.org/browse/ASH");
        testDatabase("http://jira.codehaus.org/browse/AWARE");
        testDatabase("http://jira.codehaus.org/browse/BERKANO");
        testDatabase("http://jira.codehaus.org/browse/BLISSED");
        testDatabase("http://jira.codehaus.org/browse/BOO");
        testDatabase("http://jira.codehaus.org/browse/CARGO");
    }

    public void testJBossInstance() {
        testDatabase("http://jira.jboss.com/jira/browse/JBQA");
        testDatabase("http://jira.jboss.com/jira/browse/JBWIKI");
        testDatabase("http://jira.jboss.com/jira/browse/EJBTHREE");
        testDatabase("http://jira.jboss.com/jira/browse/HIBERNATE");
        testDatabase("http://jira.jboss.com/jira/browse/JBAOP");
        testDatabase("http://jira.jboss.com/jira/browse/JBAS");
        testDatabase("http://jira.jboss.com/jira/browse/JBIDE");
        testDatabase("http://jira.jboss.com/jira/browse/JBMESSAGING");
        testDatabase("http://jira.jboss.com/jira/browse/JGRP");
        testDatabase("http://jira.jboss.com/jira/browse/JBTPL");
        testDatabase("http://jira.jboss.com/jira/browse/JBDOCS");
        testDatabase("http://jira.jboss.com/jira/browse/JASSIST");
        testDatabase("http://jira.jboss.com/jira/browse/JBADMCON");
        testDatabase("http://jira.jboss.com/jira/browse/JBBENCH");
        testDatabase("http://jira.jboss.com/jira/browse/JBBUILD");
        testDatabase("http://jira.jboss.com/jira/browse/JBCACHE");
        testDatabase("http://jira.jboss.com/jira/browse/JBCLUSTER");
        testDatabase("http://jira.jboss.com/jira/browse/JBINSTALL");
        testDatabase("http://jira.jboss.com/jira/browse/JBPM");
        testDatabase("http://jira.jboss.com/jira/browse/JBLAB");
        testDatabase("http://jira.jboss.com/jira/browse/JBMAIL");
        testDatabase("http://jira.jboss.com/jira/browse/JBMICROCONT");
        testDatabase("http://jira.jboss.com/jira/browse/JBPORTAL");
        testDatabase("http://jira.jboss.com/jira/browse/JBPROFILER");
        testDatabase("http://jira.jboss.com/jira/browse/JBREM");
        testDatabase("http://jira.jboss.com/jira/browse/JBSEAM");
        testDatabase("http://jira.jboss.com/jira/browse/JBSER");
        testDatabase("http://jira.jboss.com/jira/browse/JBSHARP");
        testDatabase("http://jira.jboss.com/jira/browse/JBWEB");
        testDatabase("http://jira.jboss.com/jira/browse/JBXB");
        testDatabase("http://jira.jboss.com/jira/browse/JBWS");
        testDatabase("http://jira.jboss.com/jira/browse/KOSMOS");
    }

    // TODO add further instances seen in the Atlassian pages

    /** Tests a database. */
    protected void testDatabase(String url) {
        List<JiraProject> projects = service.getProjects(url);
        assertTrue(projects.size() > 0);

        // test projects
        for(JiraProject project : projects) {
            assertFalse(StringUtils.isBlank(project.getName()));
            assertFalse(StringUtils.isBlank(project.getUrl()));
            assertFalse(StringUtils.isBlank(project.getKey()));
            assertFalse(StringUtils.isBlank(project.getProjectUrl()));
            assertFalse(StringUtils.isBlank(project.getDescription()));
            assertFalse(StringUtils.isBlank(project.getLead()));

            assertTrue(project.getOpenIssues() >= 0);
            assertTrue(project.getCodingInProgressIssues() >= 0);
            assertTrue(project.getReopenedIssues() >= 0);
            assertTrue(project.getResolvedIssues() >= 0);
            assertTrue(project.getClosedIssues() >= 0);

            assertTrue(project.getBlockerOpenIssues() >= 0);
            assertTrue(project.getCriticalOpenIssues() >= 0);
            assertTrue(project.getMajorOpenIssues() >= 0);
            assertTrue(project.getMinorOpenIssues() >= 0);
            assertTrue(project.getTrivialOpenIssues() >= 0);
            assertTrue(project.getOptionalOpenIssues() >= 0);

            testOpenIssuesPerAssignee(project);

            assertFalse(StringUtils.isBlank(project.getIssuesPerStatusChartLocator().getLoresUrl()));
            assertFalse(StringUtils.isBlank(project.getOpenIssuesPerPriorityChartLocator().getLoresUrl()));
            assertFalse(StringUtils.isBlank(project.getOpenIssuesPerAssigneeChartLocator().getLoresUrl()));

            assertEquals(project.getTotalIssues(), project.getOpenIssues() + project.getCodingInProgressIssues() + project.getReopenedIssues() + project.getResolvedIssues() + project.getClosedIssues());
        }
    }

    /** Tests the open issues per assignee in a project. */
    protected static void testOpenIssuesPerAssignee(JiraProject project) {
        int openIssuesCalculated = 0;
        for(Map.Entry<String, Integer> openIssuePerAssignee : project.getOpenIssuesPerAssignee()) {
            assertFalse(StringUtils.isBlank(openIssuePerAssignee.getKey()));
            assertTrue(openIssuePerAssignee.getValue() > 0);
            openIssuesCalculated += openIssuePerAssignee.getValue();
        }
        assertEquals(project.getOpenIssues(), openIssuesCalculated);
    }
}
